@@ -20,7 +20,7 @@ def cli():
 def deploy(image, namespace, network, include_hosts, envvars, cliargs):
     """Deploys NuCypher on managed hosts."""
 
-    deployer = CloudDeployers.get_deployer('generic')(emitter,
+    deployer = CloudDeployers.get_deployer('ethereum')(emitter,
                                                       docker_image=image,
                                                       namespace=namespace,
                                                       network=network,
@@ -35,4 +35,4 @@ def deploy(image, namespace, network, include_hosts, envvars, cliargs):
     for name, hostdata in [(n, d) for n, d in deployer.config['instances'].items() if n in hostnames]:
         emitter.echo(f'\t{name}: {hostdata["publicaddress"]}', color="yellow")
     os.environ['ANSIBLE_HOST_KEY_CHECKING'] = 'False'
-    deployer.deploy_image_on_existing_nodes(hostnames, 'ethereum')
+    deployer.deploy(hostnames)
